@@ -46,12 +46,12 @@ option_strings = {
     'debug_microseconds' : _('Include microseconds in timestamps in debug logs'),
     'debug_to_files' : _('Write debug messages to logfiles'),
     'timeout' : _('Watchdog timeout before restarting service'),
-    'force_timeout' : _('Timeout between three failed ping checks and forcibly killing the service'),
     'command' : _('Command to start service'),
     'reconnection_retries' : _('Number of times to attempt connection to Data Providers'),
     'fd_limit' : _('The number of file descriptors that may be opened by this responder'),
     'client_idle_timeout' : _('Idle time before automatic disconnection of a client'),
-    'diag_cmd' : _('The command to run when a service ping times out'),
+    'responder_idle_timeout' : _('Idle time before automatic shutdown of the responder'),
+    'cache_first': _('Always query all the caches before querying the Data Providers'),
 
     # [sssd]
     'services' : _('SSSD Services to start'),
@@ -63,6 +63,10 @@ option_strings = {
     'default_domain_suffix' : _('Domain to add to names without a domain component.'),
     'user' : _('The user to drop privileges to'),
     'certificate_verification' : _('Tune certificate verification'),
+    'override_space': _('All spaces in group or user names will be replaced with this character'),
+    'disable_netlink' : _('Tune sssd to honor or ignore netlink state changes'),
+    'enable_files_domain' : _('Enable or disable the implicit files domain'),
+    'domain_resolution_order': _('A specific order of the domains to be looked up'),
 
     # [nss]
     'enum_cache_timeout' : _('Enumeration cache timeout length (seconds)'),
@@ -81,13 +85,14 @@ option_strings = {
     'shell_fallback' : _('If a shell stored in central directory is allowed but not available, use this fallback'),
     'default_shell': _('Shell to use if the provider does not list one'),
     'memcache_timeout': _('How long will be in-memory cache records valid'),
-    'override_space': _('All spaces in group or user names will be replaced with this character'),
+    'user_attributes': _('List of user attributes the NSS responder is allowed to publish'),
 
     # [pam]
     'offline_credentials_expiration' : _('How long to allow cached logins between online logins (days)'),
     'offline_failed_login_attempts' : _('How many failed logins attempts are allowed when offline'),
     'offline_failed_login_delay' : _('How long (minutes) to deny login after offline_failed_login_attempts has been reached'),
     'pam_verbosity' : _('What kind of messages are displayed to the user during authentication'),
+    'pam_response_filter' : _('Filter PAM responses sent to the pam_sss'),
     'pam_id_timeout' : _('How many seconds to keep identity information cached for PAM requests'),
     'pam_pwd_expiration_warning' : _('How many days before password expiration a warning should be displayed'),
     'pam_trusted_users' : _('List of trusted uids or user\'s name'),
@@ -95,12 +100,14 @@ option_strings = {
     'pam_account_expired_message' : _('Message printed when user account is expired.'),
     'pam_account_locked_message' : _('Message printed when user account is locked.'),
     'pam_cert_auth' : _('Allow certificate based/Smartcard authentication.'),
-    'pam_cert_db_path' : _('Path to certificate databse with PKCS#11 modules.'),
+    'pam_cert_db_path' : _('Path to certificate database with PKCS#11 modules.'),
     'p11_child_timeout' : _('How many seconds will pam_sss wait for p11_child to finish'),
+    'pam_app_services' : _('Which PAM services are permitted to contact application domains'),
 
     # [sudo]
     'sudo_timed' : _('Whether to evaluate the time-based attributes in sudo rules'),
     'sudo_inverse_order' : _('If true, SSSD will switch back to lower-wins ordering logic'),
+    'sudo_threshold' : _('Maximum number of rules that can be refreshed at once. If this is exceeded, full refresh is performed.'),
 
     # [autofs]
     'autofs_negative_timeout' : _('Negative cache timeout length (seconds)'),
@@ -118,6 +125,27 @@ option_strings = {
     'allowed_uids': _('List of UIDs or user names allowed to access the InfoPipe responder'),
     'user_attributes': _('List of user attributes the InfoPipe is allowed to publish'),
 
+    # [secrets]
+    'provider': _('The provider where the secrets will be stored in'),
+    'containers_nest_level': _('The maximum allowed number of nested containers'),
+    'max_secrets': _('The maximum number of secrets that can be stored'),
+    'max_uid_secrets': _('The maximum number of secrets that can be stored per UID'),
+    'max_payload_size': _('The maximum payload size of a secret in kilobytes'),
+    # secrets - proxy
+    'proxy_url': _('The URL Custodia server is listening on'),
+    'auth_type': _('The method to use when authenticating to a Custodia server'),
+    'auth_header_name': _('The name of the headers that will be added into a HTTP request with the value defined in auth_header_value'),
+    'auth_header_value': _('The value sssd-secrets would use for auth_header_name'),
+    'forward_headers': _('The list of the headers to forward to the Custodia server together with the request'),
+    'username': _('The username to use when authenticating to a Custodia server using basic_auth'),
+    'password': _('The password to use when authenticating to a Custodia server using basic_auth'),
+    'verify_peer': _('If true peer\'s certificate is verified if proxy_url uses https protocol'),
+    'verify_host': _('If false peer\'s certificate may contain different hostname than proxy_url when https protocol is used'),
+    'capath': _('Path to directory where certificate authority certificates are stored'),
+    'cacert': _('Path to file containing server\'s CA certificate'),
+    'cert': _('Path to file containing client\'s certificate'),
+    'key': _('Path to file containing client\'s private key'),
+
     # [provider]
     'id_provider' : _('Identity provider'),
     'auth_provider' : _('Authentication provider'),
@@ -125,10 +153,12 @@ option_strings = {
     'chpass_provider' : _('Password change provider'),
     'sudo_provider' : _('SUDO provider'),
     'autofs_provider' : _('Autofs provider'),
-    'session_provider' : _('Session-loading provider'),
     'hostid_provider' : _('Host identity provider'),
+    'selinux_provider' : _('SELinux provider'),
+    'session_provider' : _('Session management provider'),
 
     # [domain]
+    'domain_type' : _('Whether the domain is usable by the OS or by applications'),
     'min_id' : _('Minimum user ID'),
     'max_id' : _('Maximum user ID'),
     'enumerate' : _('Enable enumerating all users/groups'),
@@ -161,7 +191,11 @@ option_strings = {
     'subdomain_enumerate' : _('Control enumeration of trusted domains'),
     'subdomain_refresh_interval' : _('How often should subdomains list be refreshed'),
     'subdomain_inherit' : _('List of options that should be inherited into a subdomain'),
+    'subdomain_homedir' : _('Default subdomain homedir value'),
     'cached_auth_timeout' : _('How long can cached credentials be used for cached authentication'),
+    'full_name_format' : _('Printf-compatible format for displaying fully-qualified names'),
+    're_expression' : _('Regex to parse username and domain'),
+    'auto_private_groups' : _('Whether to automatically create private groups for users'),
 
     # [provider/ipa]
     'ipa_domain' : _('IPA domain'),
@@ -182,13 +216,17 @@ option_strings = {
     'ipa_views_search_base': _("Search base for view containers"),
     'ipa_view_class': _("Objectclass for view containers"),
     'ipa_view_name': _("Attribute with the name of the view"),
-    'ipa_overide_object_class': _("Objectclass for override objects"),
+    'ipa_override_object_class': _("Objectclass for override objects"),
     'ipa_anchor_uuid': _("Attribute with the reference to the original object"),
     'ipa_user_override_object_class': _("Objectclass for user override objects"),
     'ipa_group_override_object_class': _("Objectclass for group override objects"),
+    'ipa_deskprofile_search_base': _("Search base for Desktop Profile related objects"),
+    'ipa_deskprofile_refresh': _("The amount of time in seconds between lookups of the Desktop Profile rules against the IPA server"),
+    'ipa_deskprofile_request_interval': _("The amount of time in minutes between lookups of Desktop Profiles rules against the IPA server when the last request did not find any rule"),
 
     # [provider/ad]
     'ad_domain' : _('Active Directory domain'),
+    'ad_enabled_domains' : _('Enabled Active Directory domains'),
     'ad_server' : _('Active Directory server address'),
     'ad_backup_server' : _('Active Directory backup server address'),
     'ad_hostname' : _('Active Directory client hostname'),
@@ -207,7 +245,7 @@ option_strings = {
     'ad_gpo_default_right' : _('Default logon right (or permit/deny) to use for unmapped PAM service names'),
     'ad_site' : _('a particular site to be used by the client'),
     'ad_maximum_machine_account_password_age' : _('Maximum age in days before the machine account password should be renewed'),
-    'ad_machine_account_password_renewal_opts' : _('Option for tuing the machine account renewal task'),
+    'ad_machine_account_password_renewal_opts' : _('Option for tuning the machine account renewal task'),
 
     # [provider/krb5]
     'krb5_kdcip' : _('Kerberos server address'),
@@ -231,7 +269,7 @@ option_strings = {
     'krb5_fast_principal' : _("Selects the principal to use for FAST"),
     'krb5_canonicalize' : _("Enables principal canonicalization"),
     'krb5_use_enterprise_principal' : _("Enables enterprise principals"),
-    'krb5_map_user' : _('A mapping from user names to kerberos principal names'),
+    'krb5_map_user' : _('A mapping from user names to Kerberos principal names'),
 
     # [provider/krb5/chpass]
     'krb5_kpasswd' : _('Server where the change password service is running if not on the KDC'),
@@ -312,6 +350,7 @@ option_strings = {
     'ldap_user_shadow_flag' : _('shadowFlag attribute'),
     'ldap_user_authorized_service' : _('Attribute listing authorized PAM services'),
     'ldap_user_authorized_host' : _('Attribute listing authorized server hosts'),
+    'ldap_user_authorized_rhost' : _('Attribute listing authorized server rhosts'),
     'ldap_user_krb_last_pwd_change' : _('krbLastPwdChange attribute'),
     'ldap_user_krb_password_expiration' : _('krbPasswordExpiration attribute'),
     'ldap_pwd_attribute' : _('Attribute indicating that server side password policies are active'),
@@ -324,6 +363,7 @@ option_strings = {
     'ldap_user_ssh_public_key' : _('SSH public key attribute'),
     'ldap_user_auth_type' : _('attribute listing allowed authentication types for a user'),
     'ldap_user_certificate' : _('attribute containing the X509 certificate of the user'),
+    'ldap_user_email' : _('attribute containing the email address of the user'),
 
     'ldap_user_extra_attrs' : _('A list of extra attributes to download along with the user entry'),
 
@@ -341,7 +381,7 @@ option_strings = {
     'ldap_group_type' : _('Type of the group and other flags'),
     'ldap_group_external_member' : _('The LDAP group external member attribute'),
     #replaced by ldap_entry_usn# 'ldap_group_entry_usn' : _('entryUSN attribute'),
-    'ldap_group_nesting_level' : _('Maximum nesting level SSSd will follow'),
+    'ldap_group_nesting_level' : _('Maximum nesting level SSSD will follow'),
 
     'ldap_netgroup_search_base' : _('Base DN for netgroup lookups'),
     'ldap_netgroup_object_class' : _('Objectclass for netgroups'),
@@ -425,12 +465,19 @@ option_strings = {
     'default_shell' : _('Default shell, /bin/bash'),
     'base_directory' : _('Base for home directories'),
 
+    # [provider/proxy]
+    'proxy_max_children' : _('The number of preforked proxy children.'),
+
     # [provider/proxy/id]
     'proxy_lib_name' : _('The name of the NSS library to use'),
     'proxy_fast_alias' : _('Whether to look up canonical group name from cache if possible'),
 
     # [provider/proxy/auth]
-    'proxy_pam_target' : _('PAM stack to use')
+    'proxy_pam_target' : _('PAM stack to use'),
+
+    # [provider/files]
+    'passwd_files' : _('Path of passwd file sources.'),
+    'group_files' : _('Path of group file sources.')
 }
 
 def striplist(l):
@@ -458,7 +505,7 @@ class SSSDConfigSchema(SSSDChangeConf):
             self.readfp(fd)
             fd.close()
             # Read in the provider files
-            for file in filter(lambda f: re.search('^sssd-.*\.conf$', f),
+            for file in filter(lambda f: re.search(r'^sssd-.*\.conf$', f),
                                          os.listdir(schemaplugindir)):
                 fd = open(schemaplugindir+ "/" + file)
                 self.readfp(fd)
@@ -650,7 +697,7 @@ class SSSDConfigObject(object):
 
     def get_name(self):
         """
-        Return the name of the this object
+        Return the name of the object
 
         === Returns ===
         The domain name
@@ -883,9 +930,11 @@ class SSSDService(SSSDConfigObject):
         if type(value) != option_schema[0]:
             # If it's possible to convert it, do so
             try:
-                if option_schema[0] == bool and \
-                type(value) == str:
+                if option_schema[0] == bool and type(value) == str:
                     value = self.schema.bool_lookup[value.lower()]
+                elif option_schema[0] == int and type(value) == str:
+                    # Make sure we handle any reasonable base
+                    value = int(value, 0)
                 else:
                     value = option_schema[0](value)
             except ValueError:
@@ -1352,7 +1401,7 @@ class SSSDConfig(SSSDChangeConf):
         are not passed, it will use the system defaults.
 
         schemafile:
-          The path to the api schema config file. Usually
+          The path to the API schema config file. Usually
           /usr/share/sssd/sssd.api.conf
         schemaplugindir:
           The path the directory containing the provider schema config files.
@@ -1504,7 +1553,7 @@ class SSSDConfig(SSSDChangeConf):
             # Remove any entries in this list that don't
             # correspond to an active service, for integrity
             configured_services = self.list_services()
-            for srv in service_dict.keys():
+            for srv in list(service_dict):
                 if srv not in configured_services:
                     del service_dict[srv]
 
@@ -1757,6 +1806,8 @@ class SSSDConfig(SSSDChangeConf):
         for option,value in service.get_all_options().items():
             if (type(value) == list):
                 value = ', '.join(value)
+            if option == "debug_level":
+                value = self._get_debug_level_val(value)
             addkw.append( { 'type'  : 'option',
                             'name'  : option,
                             'value' : str(value) } )
@@ -1787,7 +1838,7 @@ class SSSDConfig(SSSDChangeConf):
             # Remove any entries in this list that don't
             # correspond to an active domain, for integrity
             configured_domains = self.list_domains()
-            for dom in domain_dict.keys():
+            for dom in list(domain_dict):
                 if dom not in configured_domains:
                     del domain_dict[dom]
 
@@ -2101,6 +2152,8 @@ class SSSDConfig(SSSDChangeConf):
         for option,value in domain.get_all_options().items():
             if (type(value) == list):
                 value = ', '.join(value)
+            if option == "debug_level":
+                value = self._get_debug_level_val(value)
             self.set(sectionname, option, str(value))
 
         if domain.active:
